@@ -76,6 +76,19 @@ __all__ = [
     'CS_SUPPORT_X86_REDUCE',
     'CS_SKIPDATA_CALLBACK',
 
+    'CS_OP_INVALID',
+    'CS_OP_REG',
+    'CS_OP_IMM',
+    'CS_OP_MEM',
+    'CS_OP_FP',
+
+    'CS_GRP_INVALID',
+    'CS_GRP_JUMP',
+    'CS_GRP_CALL',
+    'CS_GRP_RET',
+    'CS_GRP_INT',
+    'CS_GRP_IRET',
+
     'CsError',
 ]
 
@@ -317,6 +330,7 @@ class _dummy_cs(object):
     def __init__(self, csh, arch):
         self.csh = csh
         self.arch = arch
+        self._detail = False
 
 
 # Quick & dirty Python function to disasm raw binary code
@@ -337,10 +351,11 @@ def cs_disasm_quick(arch, mode, code, offset, count=0):
     all_insn = ctypes.POINTER(_cs_insn)()
     res = _cs.cs_disasm(csh, code, len(code), offset, count, ctypes.byref(all_insn))
     if res > 0:
-        for i in range(res):
-            yield CsInsn(_dummy_cs(csh, arch), all_insn[i])
-
-        _cs.cs_free(all_insn, res)
+        try:
+            for i in range(res):
+                yield CsInsn(_dummy_cs(csh, arch), all_insn[i])
+        finally:
+            _cs.cs_free(all_insn, res)
     else:
         status = _cs.cs_errno(csh)
         if status != CS_ERR_OK:
@@ -377,11 +392,12 @@ def cs_disasm_lite(arch, mode, code, offset, count=0):
     all_insn = ctypes.POINTER(_cs_insn)()
     res = _cs.cs_disasm(csh, code, len(code), offset, count, ctypes.byref(all_insn))
     if res > 0:
-        for i in range(res):
-            insn = all_insn[i]
-            yield (insn.address, insn.size, insn.mnemonic.decode('ascii'), insn.op_str.decode('ascii'))
-
-        _cs.cs_free(all_insn, res)
+        try:
+            for i in range(res):
+                insn = all_insn[i]
+                yield (insn.address, insn.size, insn.mnemonic.decode('ascii'), insn.op_str.decode('ascii'))
+        finally:
+            _cs.cs_free(all_insn, res)
     else:
         status = _cs.cs_errno(csh)
         if status != CS_ERR_OK:
@@ -407,7 +423,7 @@ class CsInsn(object):
     def __init__(self, cs, all_info):
         self._raw = copy_ctypes(all_info)
         self._cs = cs
-        if self._cs._detail:
+        if self._cs._detail and self._raw.id != 0:
             # save detail
             self._detail = copy_ctypes(self._raw.detail.contents)
 
@@ -797,9 +813,11 @@ class Cs(object):
             print(code)'''
         res = _cs.cs_disasm(self.csh, code, len(code), offset, count, ctypes.byref(all_insn))
         if res > 0:
-            for i in range(res):
-                yield CsInsn(self, all_insn[i])
-            _cs.cs_free(all_insn, res)
+            try:
+                for i in range(res):
+                    yield CsInsn(self, all_insn[i])
+            finally:
+                _cs.cs_free(all_insn, res)
         else:
             status = _cs.cs_errno(self.csh)
             if status != CS_ERR_OK:
@@ -819,10 +837,12 @@ class Cs(object):
         all_insn = ctypes.POINTER(_cs_insn)()
         res = _cs.cs_disasm(self.csh, code, len(code), offset, count, ctypes.byref(all_insn))
         if res > 0:
-            for i in range(res):
-                insn = all_insn[i]
-                yield (insn.address, insn.size, insn.mnemonic.decode('ascii'), insn.op_str.decode('ascii'))
-            _cs.cs_free(all_insn, res)
+            try:
+                for i in range(res):
+                    insn = all_insn[i]
+                    yield (insn.address, insn.size, insn.mnemonic.decode('ascii'), insn.op_str.decode('ascii'))
+            finally:
+                _cs.cs_free(all_insn, res)
         else:
             status = _cs.cs_errno(self.csh)
             if status != CS_ERR_OK:
